.devel .titlebar {
  background: transparent -gtk-icontheme("system-run-symbolic") 80% 0/128px 128px no-repeat,
              linear-gradient(to left,
                              mix(@theme_selected_bg_color, @theme_bg_color, 0.5) 8%,
                              @theme_bg_color 25%);
  color: alpha(@theme_fg_color, 0.1);
}

.devel .titlebar label {
  color: @theme_fg_color;
}

.msgtags {
  margin-left: 10px;
  color: @warning_color;
}

.progress_trans {
  color: @theme_selected_bg_color;
}

.progress_fuzzy {
  color: @warning_color;
}

.progress_untranslated {
  color: @error_color;
}
